/*====================================================================

ObjectWeb Deployment Framework
Copyright (C) 2004-2006 INRIA - USTL - LIFL - GOAL
Contact: deployment@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Frdric Briclet.
Contributor(s): Eric Bruneton.

====================================================================
$Id: AbstractInstantiationTask.java,v 1.2 2006/02/16 18:13:18 merle Exp $
====================================================================*/

package org.objectweb.deployment.scheduling.component.lib;

import org.objectweb.deployment.scheduling.component.api.InstantiationTask;
import org.objectweb.deployment.scheduling.component.api.FactoryProviderTask;

/**
 * AbstractInstantiationTask implements InstantiationTask.
 *
 * Subclasses must implement:
 * - public void execute(Object context) throws Exception;
 *
 * @author <a href="mailto:Philippe.Merle@inria.fr">Philippe Merle</a>
 * @author <a href="mailto:Frdric.Briclet@lifl.fr">Frdric Briclet</a>
 *
 * @version 0.2
 */

public abstract class AbstractInstantiationTask
              extends AbstractInstanceProviderTask
           implements InstantiationTask
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the factory provider task. */
    private FactoryProviderTask factoryProviderTask_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    AbstractInstantiationTask()
    {
        // Call the default AbstractInstanceProviderTask constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.deployment.scheduling.component.api.InstantiationTask
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.deployment.scheduling.component.api.RequireFactoryProviderTask
    //
    // ==================================================================

    /**
     * Return the component factory provider task.
     *
     * @return The component factory provider task.
     */
    public FactoryProviderTask
    getFactoryProviderTask()
    {
        return factoryProviderTask_;
    }

    /**
     * Set the component factory provider task.
     *
     * @param task - The component factory provider task.
     */
    public void
    setFactoryProviderTask(FactoryProviderTask task)
    {
        if(factoryProviderTask_ != null)
            removePreviousTask(factoryProviderTask_);

        factoryProviderTask_ = task;

        if(factoryProviderTask_ != null)
            addPreviousTask(factoryProviderTask_);
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.deployment.scheduling.component.api.InstanceProviderTask
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.deployment.scheduling.core.api.Task
    //
    // ==================================================================

    /**
     * Execute the task.
     *
     * Must be implemented by subclasses.
     *
     * @param context - The context in which the task will be executed.
     *
     * @throws Exception - Thrown if a problem obscurs during execution.
     */
    // public abstract void
    // execute(Object context)
    // throws Exception;

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
