/*====================================================================

ObjectWeb Deployment Framework
Copyright (C) 2004-2006 INRIA - USTL - LIFL - GOAL
Contact: deployment@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Frdric Briclet.
Contributor(s): Eric Bruneton.

====================================================================
$Id: AbstractRequireFactoryProviderTask.java,v 1.2 2006/02/16 18:13:18 merle Exp $
====================================================================*/

package org.objectweb.deployment.scheduling.component.lib;

import org.objectweb.deployment.scheduling.component.api.RequireFactoryProviderTask;
import org.objectweb.deployment.scheduling.component.api.FactoryProviderTask;
import org.objectweb.deployment.scheduling.core.lib.AbstractTask;

/**
 * AbstractRequireFactoryProviderTask implements RequireFactoryProviderTask.
 *
 * Subclasses must implement:
 * - public void execute(Object context) throws Exception;
 * - public Object getResult();
 * - public void setResult(Object result);
 *
 * @author <a href="mailto:Philippe.Merle@inria.fr">Philippe Merle</a>
 * @author <a href="mailto:Frdric.Briclet@lifl.fr">Frdric Briclet</a>
 *
 * @version 0.2
 */

public abstract class AbstractRequireFactoryProviderTask
              extends AbstractTask
           implements RequireFactoryProviderTask
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the component factory provider task. */
    private FactoryProviderTask factoryProviderTask_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    AbstractRequireFactoryProviderTask()
    {
        // Call the default AbstractTask constructor.
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.deployment.scheduling.component.api.RequirFactoryProviderTask
    //
    // ==================================================================

    /**
     * Return the component factory provider task.
     *
     * @return The component factory provider task.
     */
    public FactoryProviderTask
    getFactoryProviderTask()
    {
        return factoryProviderTask_;
    }

    /**
     * Set the component factory provider task.
     *
     * @param task - The component factory provider task.
     */
    public void
    setFactoryProviderTask(FactoryProviderTask task)
    {
        if(factoryProviderTask_ != null)
            removePreviousTask(factoryProviderTask_);

        factoryProviderTask_ = task;

        if(factoryProviderTask_ != null)
            addPreviousTask(factoryProviderTask_);
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.deployment.scheduling.core.api.Task
    //
    // ==================================================================

    /**
     * Execute the task.
     *
     * Must be implemented by subclasses.
     *
     * @param context - The context in which the task will be executed.
     *
     * @throws Exception - Thrown if a problem obscurs during execution.
     */
    // public abstract void
    // execute(Object context)
    // throws Exception;

    /**
     * Return the result of the execution.
     *
     * Must be implemented by subclasses.
     *
     * @return The result of the execution.
     */
    // public abstract Object
    // getResult();

    /**
     * Set the result of the execution.
     *
     * Must be implemented by subclasses.
     *
     * @param result - The result of the execution.
     *
     * TODO: Is it really useful to provide this method?
     */
    // public abstract void
    // setResult(Object result);

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
