/*====================================================================

ObjectWeb Deployment Framework
Copyright (C) 2004-2006 INRIA - USTL - LIFL - GOAL
Contact: deployment@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Frdric Briclet.
Contributor(s): Eric Bruneton.

====================================================================
$Id: Task.java,v 1.2 2006/02/16 18:13:18 merle Exp $
====================================================================*/

package org.objectweb.deployment.scheduling.core.api;

/**
 * Task is the base interface of any deployment task to schedule.
 *
 * A Task could be executed by a Scheduler when its previous
 * tasks have been already executed by the Scheduler.
 *
 * This Task interface should be specialized by inheritance.
 *
 * @author <a href="mailto:Philippe.Merle@inria.fr">Philippe Merle</a>
 * @author <a href="mailto:Frdric.Briclet@lifl.fr">Frdric Briclet</a>
 *
 * @version 0.2
 */

public interface Task
{
    /**
     * Add a previous task.
     *
     * @param task - The previous task to add.
     */
    public void
    addPreviousTask(Task task);

    /**
     * Remove a previous task.
     *
     * @param task - The previous task to remove.
     */
    public void
    removePreviousTask(Task task);

    /**
     * Return all the previous tasks.
     *
     * @return All the previous tasks.
     */
    public Task[]
    getPreviousTasks();

    /**
     * Execute the task.
     *
     * @param context - The context in which the task will be executed.
     *
     * @throws Exception - Thrown if a problem obscurs during execution.
     */
    public void
    execute(Object context)
    throws Exception;

    /**
     * Return the result of the execution.
     *
     * @return The result of the execution.
     */
    public Object
    getResult();

    /**
     * Set the result of the execution.
     *
     * @param result - The result of the execution.
     *
     * TODO: Is it really useful to provide this method?
     */
    public void
    setResult(Object result);
}
