/*====================================================================

ObjectWeb Deployment Framework
Copyright (C) 2004-2006 INRIA - USTL - LIFL - GOAL
Contact: deployment@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Frdric Briclet.
Contributor(s): Eric Bruneton.

====================================================================
$Id: BasicScheduler.java,v 1.2 2006/02/16 18:13:18 merle Exp $
====================================================================*/

package org.objectweb.deployment.scheduling.core.lib;

import org.objectweb.deployment.scheduling.core.api.Scheduler;
import org.objectweb.deployment.scheduling.core.api.Task;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * BasicScheduler implements Scheduler.
 *
 * @author <a href="mailto:Philippe.Merle@inria.fr">Philippe Merle</a>
 * @author <a href="mailto:Frdric.Briclet@lifl.fr">Frdric Briclet</a>
 *
 * @version 0.2
 */

public class BasicScheduler
  implements Scheduler
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    BasicScheduler()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Prepare scheduling.
     *
     * TODO: Must be optimized and must detect cycle dependencies between tasks.
     *
     * @param tasks - The tasks to schedule.
     *
     * @return The scheduled tasks.
     */
    protected Task[]
    prepareScheduling(Task[] tasks)
    {
       ArrayList result = new ArrayList();

       HashMap allNextTasks = new HashMap();
       HashMap allPreviousTasks = new HashMap();

       for(int i=0; i<tasks.length; i++)
       {
           Task currentTask = tasks[i];

           allNextTasks.put(currentTask, new ArrayList());
           ArrayList tl = new ArrayList();
           Task[] previousTasks = currentTask.getPreviousTasks();
           for(int j=0; j<previousTasks.length; j++)
           {
               tl.add(previousTasks[j]);
           }
           allPreviousTasks.put(currentTask, tl);
       }

       for(int i=0; i<tasks.length; i++)
       {
           Task currentTask = tasks[i];

           Task[] previousTasks = currentTask.getPreviousTasks();
           for(int j=0; j<previousTasks.length; j++)
           {
               ArrayList tl = (ArrayList)allNextTasks.get(previousTasks[j]);
               tl.add(currentTask);
           }
       }

       while(allPreviousTasks.size() != 0)
       {
           for(int i=0; i<tasks.length; i++)
           {
               Task currentTask = tasks[i];
               ArrayList previousTaskList = (ArrayList)allPreviousTasks.get(currentTask);
               if(previousTaskList != null && previousTaskList.size() == 0)
               {
                   result.add(currentTask);
                   ArrayList al = (ArrayList)allNextTasks.get(currentTask);
                   Task[] nextTasks = (Task[])al.toArray(new Task[al.size()]);
                   for(int j=0; j<nextTasks.length; j++)
                   {
                       previousTaskList = (ArrayList)allPreviousTasks.get(nextTasks[j]);
                       previousTaskList.remove(currentTask);
                   }
                   allPreviousTasks.remove(currentTask);
               }
           }
       }

       return (Task[])result.toArray(new Task[result.size()]);
    }

    /**
     * Schedule an array of tasks.
     *
     * @param tasks - The array of tasks to schedule.
     * @param context - The execution context.
     */
    protected void
    doSchedule(Task[] tasks,
             Object context)
    {
       for(int i=0; i<tasks.length; i++)
       {
           Task currentTask = tasks[i];
           try
	   {
               currentTask.execute(context);
           }
	   catch(Exception exc)
	   {
               exc.printStackTrace();
	   }
       }
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.deployment.scheduling.core.api.Scheduler
    //
    // ==================================================================

    /**
     * Schedule an array of tasks.
     *
     * @param tasks - The array of tasks to schedule.
     */
    public void
    schedule(Task[] tasks, Object context)
    {
       doSchedule(prepareScheduling(tasks), context);
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
